####################################################################
#    CMFSubversionBrowser
#    Copyright (C) 2002-2004 kapil thangavelu <k_vertigo@objectrealms.net>
#
#    This product is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This product is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this product; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
####################################################################
"""
Purpose: Allow script access to guess content type method from OFS
Author: kapil thangavelu <k_vertigo@objectrealms.net>
$Id: CMFSubversionBrowser.py,v 1.3 2003/02/28 05:10:01 k_vertigo Exp $
"""

import itertools
from cStringIO import StringIO

from svn import fs, util

from AccessControl.SecurityInfo import ModuleSecurityInfo
from OFS.content_types import guess_content_type


class RevisionMapGenerator:
    
    """
    A class used for generating the list maps showing a bit more graphically
    the changes for a revision. Its a blatant violation of good separation,
    but it works ;-) inspired by cia.navi.cx
    """

    _textModImage = '<img src="svn_rev_text_change.png" alt="Text Modified"/>'
    _propModImage = '<img src="svn_rev_prop_change.png" alt="Properties Modified"/>'

    _imageMap = {
        0 : '<img src="svn_rev_node_modified.png" alt="Node Modified"/>',
        1 : '<img src="svn_rev_node_add.png" alt="Node Added"/>',
        2 : '<img src="svn_rev_node_delete.png" alt="Node Deleted"/>',
        3 : '<img src="svn_rev_node_replaced.png" alt="Node Replaced"/>'
        }
    
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, svn_ctx, info, revision):
        self.svn_ctx = svn_ctx
        self.resource_ctx = self.svn_ctx.getResourceContext()
        self.info = info
        self.revision = revision

    def __call__(self):
        self.pool = util.svn_pool_create( self.resource_ctx.pool )
        try:
            paths = self.info.paths.keys()
            paths = self.sort_paths( paths )
            return self.stack_draw( paths )
        finally:
            util.svn_pool_clear( self.pool )
            util.svn_pool_destroy( self.pool )
        
    def sort_paths(self, paths):
        dsu_paths = [p.split('/') for p in paths]
        dsu_paths.sort()
        paths = ['/'.join(dsu) for dsu in dsu_paths]
        return paths

    def stack_draw(self, paths ):
        out = StringIO()
        stack = []

        print >> out, '<ul class="svn_file_tree">',

        for p in paths:
            
            isdir = fs.is_dir( self.resource_ctx.fsroot, p, self.pool)
            
            segments = p.split('/')
            segments = filter(None, segments)
            segments_mapped = zip(itertools.count(), segments)

            segment_len = len(segments)
            stack_len = len(stack)

            for idx, segment in segments_mapped:
                elements = []
                if idx+1 <= stack_len: # check whats on the stack
                    if stack[idx][0] != segment: # segments don't match
                        chafe = stack[idx:]
                        chafe.reverse()
                        for item in chafe:
                            for el in item[1]:
                                print >> out, el,
                        stack = stack[:idx]
                        stack_len = len(stack)
                    elif idx+1<segment_len:
                        # if there is more to come then we're on a directory
                        # and we have contents insure that we've started the
                        # proper tree below.
                        if not "</ul>" in stack[idx][1]:
                            print >> out, '<ul class="svn_file_tree">',
                            stack[idx][1].insert(0,"</ul>")
                        continue

                if idx + 1 < segment_len: # more to come its a directory
                    print >> out, '<li class="svn_directory">',
                    elements.append('</li>')
                    print >> out, segment,
                    print >> out, '<ul class="svn_file_tree">',
                    elements.append('</ul>')

                else: # end note check if its a
                    url = self.svn_ctx.formatRevPath( p, self.revision)
                    if isdir: # directory
                        print >> out, '<li class="svn_directory">',
                        print >> out, '<a href="%s">%s</a>'%(url, segment),
                        elements.append('</li>')
                    else: # or file
                        print >> out, '<li class="svn_file">',
                        print >> out, '<a href="%s">%s</a>'%(url, segment),

                    # print change descriptors
                    change_desc = self.info.paths[p]
                    change_image = self._imageMap.get(change_desc.change_kind, '')
                    print >> out, change_image,
                    if change_desc.text_mod:
                        print >> out, self._textModImage,
                    if change_desc.prop_mod:
                        print >> out, self._propModImage,

                    if not isdir: # finish files
                        print >> out, '</li>',
                elements.reverse()
                if elements:
                    stack.append( ( segment, elements ) )
                    stack_len = len(stack)
                    
        # clear off the stack
        stack.reverse()
        for sel in stack:
            for el in sel[1]:
                print >> out, el,

        print >> out, '</ul>',

        # return stream
        return out.getvalue()


security = ModuleSecurityInfo('Products.CMFSubversionBrowser.Script')
security.declarePublic('guess_content_type')
security.declarePublic('RevisionMapGenerator')
security.apply(globals())
