#################################
#    CMFSubversionBrowser
#    Copyright (C) 2003  kapil thangavelu
#
#    This product is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This product is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this product; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#################################

"""
Purpose: CMF variant of SVN Browser object model

 This module handles all the zope related traversal
 hooks needed by the subversion browser. the traversal
 class below is a mixin for the subversion browser/mount
 point.

 bobotraverse on nodes would imply a large cost due to zpt traversal
 this impl, tries to minimize by restricting the bobotraverse impl
 to the mount point and trying to insure that actual svn traversal
 only happens during initial publisher traversal.
 
Author: kapil thangavelu <pluto@objectrealms.net>
$Id: Traversal.py,v 1.3 2003/02/28 05:15:57 k_vertigo Exp $
"""

from svn.core import SubversionException

from Shared.DC.ZRDB.TM import TM
from Products.CMFCore.utils import getActionContext

import SubversionBrowser as svnb
from utils import svn_path_join

MAGIC_PATH_PREFIX = 'svntmagic'

class SubversionZopeTraversal:
    _v_browser_traverse = None

    def __bobo_traverse__(self, REQUEST, name=None):

        obj = None
        
        if not self._v_browser_traverse and self.getTraverserSubPath() and self.repository_path:
            # force use of latest revision available from repository unless an
            # explict repository version is specified
            revision = REQUEST.get('revision')
            if revision is not None and not isinstance( revision, int):
                try: revision = int( revision )
                except: revision = None
            self.setRevision( revision )
            
            obj = self            
            collected = []
            svnpath = self.svn_path
            finished = 0
            segments = self.getTraverserSubPath()
            
            svnb.log.debug('testing %s'%(str(segments)))
            
            while 1:
                rs = segments.pop(0)
                if rs.startswith(MAGIC_PATH_PREFIX):
                    s = rs[len(MAGIC_PATH_PREFIX):]
                else:
                    s = rs
                svnb.log.debug('path segment %s'%s)
                svnb.log.debug('remaining segments %s'%str(segments))
                
                if not segments:
                    #svnb.log.debug('current actions %s'%(getActionPaths(obj)))
                    if s in getActionPaths(obj):
                        break
                               
                next_svnpath = svn_path_join(svnpath, s)

                try:                    
                    next = self.getSVNObject(next_svnpath).__of__(obj)
                except SubversionException, e:
                    break

                svnb.log.debug('got object %s'%next_svnpath)
                
                collected.append(next)
                if obj is not self:
                    setattr(obj, rs, next)
                svnpath = next_svnpath
                obj = next

                if not segments:
                    finished = 1
                    break

            TransactionalAttributes(self, _v_browser_traverse=None).register()
            
            if collected:        
                obj = collected.pop(0) 
            else:
                obj = None

            del collected

            try:
                svnb.log.debug('returning %s'%str(obj.getId()))
            except:
                pass
            self._v_browser_traverse = obj or 1

        svnb.log.debug('traversing to %s'%name)

        if obj is None: 
            if self._v_browser_traverse and not isinstance(self._v_browser_traverse, int) \
               and self._v_browser_traverse.getId() == name:
                obj = self._v_browser_traverse
            else:
                obj = getattr(self, name)
        return obj

    def __before_publishing_traverse__(self, self2, request):
        path = request['TraversalRequestNameStack']
        # alter the path to allow for '_' prefixed paths by munging
        # before actual traversal.
        altered_path = []
        for p in path:
            if p.startswith('_'):
                altered_path.append('%s%s'%(MAGIC_PATH_PREFIX, p))
            else:
                altered_path.append(p)
        request['TraversalRequestNameStack'] = altered_path
        subpath = altered_path[:]
        subpath.reverse()
        request.set('traverse_subpath', subpath)

    def getTraverserSubPath(self):
        if hasattr(self, 'REQUEST'):
            return self.REQUEST.other.get('traverse_subpath', [])
        else:
            return []

class TransactionalAttributes(TM):

    def __init__(self, object, **attr_names):
        self.object = object

        attrs = []
        methods = []
        for k,v in attr_names.items():
            if v is None:
                attrs.append( k )
            else:
                methods.append( k )
        self.attr_names = attrs
        self.method_names = methods
        
    def clear(self):
        for an in self.attr_names:
            try:
                del self.object.__dict__[an]
            except:
                pass

        for mn in self.method_names:
            try:
                getattr(self.object, mn)()
            except:
                pass
 
    _finish = _abort  = clear
    register = TM._register

    # 2.6.0-2.6.1.b1 fix
    def sortKey(self, *ignored): return 1

def getActionPaths(obj):
    return  [a.action for a in obj.getTypeInfo().listActions()]


# TypeInfo Actions are now Real Actions, its hardly worth the trouble
# for our purposes.

def getActionPaths(obj):
    ec = getActionContext( obj )
    action_urls = [a.action(ec).strip() for a in obj.getTypeInfo().listActions()]
    res = []
    for au in action_urls:
        if au[0] == '/':
            res.append( au[1:] )
        else:
            res.append( au )
    return res
    
