import os, sys, md5
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
    
from unittest import TestSuite, makeSuite, TestCase

from Products.CMFSubversionBrowser.SubversionBrowser import SubversionContext
from Products.CMFSubversionBrowser import ProductHome

test_home = os.path.join( ProductHome, 'tests')
test_repo_path = os.path.join( test_home, 'testrepo')

def print_tree(d, indent=1):
    print ' '*(indent-2), d.getId()
    for f in d.getFiles():
        print ' '*indent, f.getId()
    print 
    for sd in d.getDirectories():
        print_tree(sd, indent+2)
        
class TestSVNCore( TestCase ):

    def setUp( self ):
        self.browser = SubversionContext('browser', test_repo_path, '/core')
        self.root = self.browser.getSVNRootObject()
        
    def tearDown( self ):
        del self.root
        self.browser.clear()
        del self.browser
        
    def test_getFiles(self):
        files = self.root.getFiles()
        self.assertEqual( len(files), 2)
        file_set_head = ['cats.txt', 'elephants.txt']
        for f in files:
            truth = f.getId() in file_set_head
            self.assertEqual(truth, 1)

    def test_getFilesInitialRevision(self):
        self.browser.initialize( revision=1 )
        self.root = self.browser.getSVNRootObject()
        files = self.root.getFiles()
        self.assertEqual( len(files), 3)
        file_set_head = ['rabbits.txt', 'svn_file.png','elephants.txt']
        for f in files:
            truth = f.getId() in file_set_head
            self.assertEqual(truth, 1)

    def test_nodeProperties(self):
        file = self.root['elephants.txt']
        propval = file.getProperty('svn:keywords')
        self.assertEqual(propval, 'Id')

        file = self.root['resources']['icon.png']
        propval = file.getProperty('svn:mime-type')
        self.assertNotEqual(propval, None)

    def test_nodeLog(self):
        entries = self.root['elephants.txt'].getLogEntries()
        self.assertEqual(len(entries), 3 )
        entries = self.root['cats.txt'].getLogEntries()
        revisions = (1,2,3)
        for i in entries:
            self.assertEqual( i.author, 'hazmat')
            assert i.revision in revisions
            
        self.assertEqual(len(entries), 3)

    def test_getDirectories(self):
        self.assertEqual( len(self.root.getDirectories()), 1)

    def test_RevisionPathMap( self ):
        rev_map = self.root['resources']['icon.png'].getRevisionPathMap(cross_copies=1)

        map = { 1:'/core/svn_file.png',
                3:'/core/icon.png',
                5:'/core/resources/icon.png' }

        self.assertEqual(len(rev_map), 3)
        for mk in map.keys():
            assert mk in rev_map
            self.assertEqual( map[mk], rev_map[mk] )
        
    def test_Annotation(self):
        file = self.root['elephants.txt']
        lines =  file.getAnnotatedLines()
        line_rev_map = (4, 1, 2)
        for i in range(len(lines)):
            self.assertEqual( line_rev_map[i], lines[i][0])

    def Xtest_getSize(self):
        file = self.root['elephants.txt']
        size = file.getSize()
        print 'Size', size

    def test_getLog(self):
        file = self.root['elephants.txt']
        log = file.getLog()
        self.assertEqual(log.author, 'hazmat')
        self.assertEqual(log.message, 'add props and line\n')
        
    def Xtest_getMD5(self):
        file = self.root['elephants.txt']

        checksum = file.getMD5()
        print 'sums2', checksum
        contents = file.getContents()
        md5sink = md5.new()
        md5sink.update( contents )
        checksum2 = md5sink.hexdigest()
        self.assertEqual( checksum, checksum2)
        
    def test_setRevision(self):
        self.browser.initialize( revision=1 )
        # now test that a call to set revision with no argswill reset us to the latest
        self.browser.setRevision( None )
        root = self.browser.getSVNRootObject()
        self.assertEqual( len(root.getFiles()), 2)

    def test_getRevisionInfo(self):
        info = self.browser.getRevisionInfo()
        self.assertEqual( len(info.paths), 3)


if __name__ == '__main__':
    framework()
else:
    
    def test_suite():
        suite = TestSuite()
        suite.addTest(makeSuite(TestSVNCore))
        return suite    
    


        
